'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var redux = require('redux');
var createSelector = require('create-selector');
var createRouteMatcher = _interopDefault(require('feather-route-matcher'));
var reselect = require('reselect');
var idbKeyval = _interopDefault(require('idb-keyval'));
var ric = _interopDefault(require('ric-shim'));
var debounce = _interopDefault(require('lodash/debounce'));
var URL = _interopDefault(require('url-parse'));
var qs = _interopDefault(require('querystringify'));

var appTimeBundle$1 = {
  name: 'appTime',
  reducer: Date.now,
  selectAppTime: function (state) { return state.appTime; }
}

var changes = {
  'START': 1,
  'SUCCESS': -1,
  'ERROR': -1
}

var asyncCount = {
  name: 'asyncCount',
  reducer: function (state, ref) {
    if ( state === void 0 ) state = 0;
    var type = ref.type;

    var last = type.split('_').slice(-1)[0]
    return state + (changes[last] || 0)
  },
  selectAsyncActive: function (state) { return state.asyncCount > 0; }
}

var debugMiddleware = function (store) { return function (next) { return function (action) {
  var isDebug = store.getState().debug

  if (isDebug) {
    console.group(action.type)
    console.info('action:', action)
  }

  var result = next(action)

  if (isDebug) {
    console.debug('state:', store.getState())
    self.logSelectors && self.logSelectors()
    self.logNextReaction && self.logNextReaction()
    console.groupEnd(action.type)
  }

  return result
}; }; }

function createThunkMiddleware (extra) {
  return function (ref) {
    var dispatch = ref.dispatch;
    var getState = ref.getState;
    var store = ref.store;

    return function (next) { return function (action) {
    if (typeof action === 'function') {
      return action(Object.assign({}, store, extra))
    }
    return next(action)
  }; };
  }
}

var thunk = createThunkMiddleware()
thunk.withExtraArgs = createThunkMiddleware

function applyMiddleware () {
  var middlewares = [], len = arguments.length;
  while ( len-- ) middlewares[ len ] = arguments[ len ];

  return function (createStore) { return function (reducer, preloadedState, enhancer) {
    var store = createStore(reducer, preloadedState, enhancer)
    var dispatch = store.dispatch
    var chain = []

    var middlewareAPI = {
      getState: store.getState,
      dispatch: function (action) { return dispatch(action); },
      store: store
    }
    chain = middlewares.map(function (middleware) { return middleware(middlewareAPI); })
    dispatch = redux.compose.apply(redux, chain)(store.dispatch)

    return Object.assign(store, { dispatch: dispatch })
  }; }
}

var consumeBundle = function (bundle, accum, bundles) {
  if ( accum === void 0 ) accum = {};

  // simple test to see whether we have needed props
  if (!accum.reducers) {
    Object.assign(accum, {
      reducers: {},
      extraMiddleware: {},
      initMethods: {},
      extraArgs: {},
      itemsToExtract: {},
      actionCreators: {},
      selectors: {}
    })
  }
  var name = bundle.name;
  Object.keys(bundle).forEach(function (key) {
    var value = bundle[key]
    if (key === 'reducer') {
      accum.reducers[name] = value
      return
    }
    if (key === 'getReducer') {
      accum.reducers[name] = value()
      return
    }
    if (key === 'getMiddleware') {
      accum.extraMiddleware[name] = value
      return
    }
    if (key === 'init') {
      accum.initMethods[name] = value
      return
    }
    if (key === 'extraArgs') {
      Object.assign(accum.extraArgs, value)
      return
    }
    if (key === 'extract') {
      accum.itemsToExtract[name] = bundles.reduce(function (accum, bundle) {
        var extracted = bundle[value]
        if (extracted) {
          accum[bundle.name] = extracted
        }
        return accum
      }, {})
    }
    if (key.indexOf('do') === 0) {
      var obj = {}
      obj[key] = value
      Object.assign(accum.actionCreators, obj)
      return
    }
    if (key.indexOf('select') === 0) {
      var obj$1 = {}
      obj$1[key] = value
      Object.assign(accum.selectors, obj$1)
      return
    }
  })
  return accum
}

var bindSelectorsToStore = function (store, selectors) {
  var loop = function ( key ) {
    var selector = selectors[key]
    store[key] = function () { return selector(store.getState()); }
  };

  for (var key in selectors) loop( key );
}

var decorateStore = function (store, meta) {
  store.meta || (store.meta = {})
  var combinedSelectors = Object.assign(store.meta.selectors || {}, meta.selectors)
  createSelector.resolveSelectors(combinedSelectors)
  store.meta.selectors = combinedSelectors
  bindSelectorsToStore(store, combinedSelectors)
  Object.assign(store, redux.bindActionCreators(meta.actionCreators, store.dispatch))
  Object.assign(store.meta, meta)
  for (var appName in meta.initMethods) {
    meta.initMethods[appName](store, meta.itemsToExtract[appName])
  }
}

var composeBundles$1 = function () {
  var bundles = [], len = arguments.length;
  while ( len-- ) bundles[ len ] = arguments[ len ];

  // build out object of extracted bundle info
  var meta = {}
  bundles.forEach(function (bundle) { return consumeBundle(bundle, meta, bundles); })
  return function (data) {
    var middleware = [
      thunk.withExtraArgs(meta.extraArgs),
      debugMiddleware
    ]

    for (var appName in meta.extraMiddleware) {
      middleware.push(meta.extraMiddleware[appName](meta.itemsToExtract[appName]))
    }

    var store = redux.createStore(
      redux.combineReducers(meta.reducers),
      data,
      applyMiddleware.apply(void 0, middleware)
    )

    store.bundles = bundles

    decorateStore(store, meta)

    store.integrateBundles = function () {
      var bundlesToIntegrate = [], len = arguments.length;
      while ( len-- ) bundlesToIntegrate[ len ] = arguments[ len ];

      var bundles = store.bundles;
      var bundleMap = bundles.reduce(function (acc, bundle, index) {
        acc[bundle.name] = index
        return acc
      }, {})
      bundlesToIntegrate.forEach(function (newBundle) {
        var currentIndex = bundleMap[newBundle.name]
        if (currentIndex != null) {
          bundles[currentIndex] = newBundle
        } else {
          bundles.push(newBundle)
        }
      })
      var meta = {}
      bundles.forEach(function (bundle) { return consumeBundle(bundle, meta, bundles); })
      decorateStore(store, meta)
      store.replaceReducer(redux.combineReducers(meta.reducers))
    }

    return store
  }
}

var createRoutingBundle = function (routes) { return ({
  name: 'routes',
  selectRouteInfo: createSelector.createSelector('selectPathname', createRouteMatcher(routes)),
  selectRouteParams: createSelector.createSelector('selectRouteInfo', function (ref) {
    var params = ref.params;

    return params;
  }),
  selectCurrentComponent: createSelector.createSelector('selectRouteInfo', function (ref) {
    var page = ref.page;

    return page;
  })
}); }

var debug = false
try { debug = !!window.localStorage.debug } catch (e) {}
var HAS_DEBUG_FLAG = debug
var HAS_WINDOW = typeof window !== 'undefined'
var IS_BROWSER = HAS_WINDOW || typeof self !== 'undefined'
var flattenExtractedToObject = function (extracted) {
  var result = {}
  for (var appName in extracted) {
    Object.assign(result, extracted[appName])
  }
  return result
}
var flattenExtractedToArray = function (extracted) {
  var accum = []
  for (var appName in extracted) {
    accum.push.apply(accum, extracted[appName])
  }
  return accum
}
var addGlobalListener = function (eventName, handler) {
  if (IS_BROWSER) {
    self.addEventListener(eventName, handler)
  }
}
var selectorNameToValueName = function (name) { return name.charAt(6).toLowerCase() + name.slice(7); }

var OFFLINE = 'OFFLINE'
var ONLINE = 'ONLINE'

var onlineBundle$1 = {
  name: 'online',
  selectIsOnline: function (state) { return state.online; },
  reducer: function (state, ref) {
    if ( state === void 0 ) state = true;
    var type = ref.type;

    if (type === OFFLINE) return false
    if (type === ONLINE) return true
    return state
  },
  init: function (store) {
    addGlobalListener('online', function () { return store.dispatch({type: ONLINE}); })
    addGlobalListener('offline', function () { return store.dispatch({type: OFFLINE}); })
  }
}

var createTimeCheckSelector = function (timeSelector, age, invert) {
  if ( invert === void 0 ) invert = true;

  return reselect.createSelector(
  timeSelector,
  appTimeBundle$1.selectAppTime,
  function (time, appTime) {
    if (!time) {
      return false
    }
    var elapsed = appTime - time
    if (invert) {
      return elapsed > age
    } else {
      return elapsed < age
    }
  }
);
}

var defaultOpts = {
  name: null,
  getPromise: null,
  actionBaseType: null,
  staleAge: 900000, // fifteen minutes
  retryAfter: 60000, // one minute,
  checkIfOnline: true,
  persist: true
}

var createAsyncResourceBundle$1 = function (spec) {
  var opts = Object.assign({}, defaultOpts, spec)
  var name = opts.name;
  var staleAge = opts.staleAge;
  var retryAfter = opts.retryAfter;
  var actionBaseType = opts.actionBaseType;
  var checkIfOnline = opts.checkIfOnline;
  var ucaseName = name.charAt(0).toUpperCase() + name.slice(1)

  if (process.env.NODE_ENV !== 'production') {
    for (var item in opts) {
      if (opts[item] === null) {
        throw Error(("You must supply an " + item + " option when creating a resource bundle"))
      }
    }
  }

  // build selectors
  var inputSelector = function (state) { return state[name]; }
  var dataSelector = reselect.createSelector(
    inputSelector,
    function (resourceState) { return resourceState.data; }
  )
  var lastSuccessSelector = reselect.createSelector(inputSelector, function (resource) { return resource.lastSuccess; })
  var lastErrorSelector = reselect.createSelector(
    inputSelector,
    function (resource) { return resource.errorTimes.slice(-1)[0] || null; }
  )
  var isStaleSelector = createTimeCheckSelector(lastSuccessSelector, staleAge)
  var isWaitingToRetrySelector = createTimeCheckSelector(lastErrorSelector, retryAfter, false)
  var isLoadingSelector = reselect.createSelector(
    inputSelector,
    function (resourceState) { return resourceState.isLoading; }
  )
  var failedPermanentlySelector = reselect.createSelector(
    inputSelector,
    function (resourceState) { return resourceState.failedPermanantly; }
  )
  var shouldUpdateSelector = reselect.createSelector(
    isLoadingSelector,
    failedPermanentlySelector,
    isWaitingToRetrySelector,
    dataSelector,
    isStaleSelector,
    onlineBundle$1.selectIsOnline,
    function (isLoading, failedPermanantly, isWaitingToRetry, data, isStale, isOnline) {
      if (
        (checkIfOnline && !isOnline) ||
        isLoading ||
        failedPermanantly ||
        isWaitingToRetry
      ) {
        return false
      }
      if (!data) {
        return true
      }
      return isStale
    }
  )

  var actions = {
    START: (actionBaseType + "_START"),
    SUCCESS: (actionBaseType + "_SUCCESS"),
    ERROR: (actionBaseType + "_ERROR")
  }

  var initialState = {
    isLoading: false,
    data: null,
    errorTimes: [],
    lastSuccess: null,
    failedPermanantly: false
  }

  var doFetchError = function (error) { return ({type: actions.ERROR, error: error}); }
  var doFetchSuccess = function (payload) { return ({type: actions.SUCCESS, payload: payload}); }
  var doFetchData = function () { return function (args) {
    var dispatch = args.dispatch;
    dispatch({type: actions.START})
    return opts.getPromise(args)
      .then(
        function (payload) { dispatch(doFetchSuccess(payload)) },
        function (error) { dispatch(doFetchError(error)) }
      )
  }; }

  var result = {
    name: name,
    reducer: function (state, ref) {
      if ( state === void 0 ) state = initialState;
      var type = ref.type;
      var payload = ref.payload;
      var error = ref.error;
      var merge = ref.merge;

      if (type === actions.START) {
        return Object.assign({}, state, { isLoading: true })
      }
      if (type === actions.SUCCESS) {
        var updatedData
        if (merge) {
          updatedData = Object.assign({}, state.data, payload)
        } else {
          updatedData = payload
        }
        return Object.assign({}, state, {
          isLoading: false,
          data: updatedData,
          lastSuccess: Date.now(),
          errorTimes: [],
          failedPermanantly: false
        })
      }
      if (type === actions.ERROR) {
        return Object.assign({}, state, {
          isLoading: false,
          errorTimes: state.errorTimes.concat([Date.now()]),
          failedPermanantly: !!(error && error.permanent)
        })
      }
      return state
    }
  };
  result[("select" + ucaseName + "Raw")] = inputSelector;
  result[("select" + ucaseName)] = dataSelector;
  result[("select" + ucaseName + "IsStale")] = isStaleSelector;
  result[("select" + ucaseName + "LastError")] = lastErrorSelector;
  result[("select" + ucaseName + "IsWaitingToRetry")] = isWaitingToRetrySelector;
  result[("select" + ucaseName + "IsLoading")] = isLoadingSelector;
  result[("select" + ucaseName + "FailedPermanantly")] = failedPermanentlySelector;
  result[("select" + ucaseName + "ShouldUpdate")] = shouldUpdateSelector;
  result[("doFetch" + ucaseName)] = doFetchData;
  result[("doFetchSuccess" + ucaseName)] = doFetchSuccess;
  result[("doFetchError" + ucaseName)] = doFetchError

  if (opts.persist) {
    result.persistActions = [ actions.SUCCESS ]
  }

  return result
}

// Thin layer on top of idbkey val with support for versioning and
// max age
var defaultOpts$1 = {maxAge: Infinity, version: 0, lib: idbKeyval}

var getCachedItem = function (key, opts) {
  var ref = Object.assign(opts, defaultOpts$1);
  var maxAge = ref.maxAge;
  var version = ref.version;
  var lib = ref.lib;
  return lib.get(key)
    .then(JSON.parse)
    .then(function (parsed) {
      var age = Date.now() - parsed.time
      if (age > maxAge || version !== parsed.version) {
        lib.delete(key)
        return null
      }
      return {
        age: age,
        data: parsed.data
      }
    })
    .catch(function () { return null; })
}

var getAllCached = function (opts) {
  Object.assign(opts, defaultOpts$1)
  var keys
  return opts.lib.keys()
    .then(function (retrivedKeys) {
      keys = retrivedKeys
      return Promise.all(keys.map(function (key) { return getCachedItem(key, opts)
          .then(function (res) { return res.data; }); }
      ))
    })
    .then(function (data) { return data.reduce(function (acc, bundleData, index) {
      if (bundleData) {
        acc[keys[index]] = bundleData
      }
      return acc
    }, {}); })
    .catch(function () {})
}

var clearAllCached = function (opts) {
    if ( opts === void 0 ) opts = defaultOpts$1;

    return opts.lib.clear().catch(function () { return null; });
}

var cacheItem = function (key, data, opts) {
  Object.assign(opts, defaultOpts$1)
  return opts.lib.set(key, JSON.stringify({
    version: opts.version,
    time: Date.now(),
    data: data
  }))
  .catch(function () { return null; })
}

var defaults = { version: 0 }
var caching = function (spec) {
  var opts = Object.assign({}, defaults, spec)

  return {
    name: 'localCache',
    extract: 'persistActions',
    getMiddleware: function (extracted) {
      var combinedActions = {}
      var loop = function ( bundleName ) {
        (extracted[bundleName] || []).forEach(function (type) {
          combinedActions[type] || (combinedActions[type] = [])
          combinedActions[type].push(bundleName)
        })
      };

      for (var bundleName in extracted) loop( bundleName );

      return function (ref) {
        var getState = ref.getState;

        return function (next) { return function (action) {
        var keys = combinedActions[action.type]
        var res = next(action)
        var state = getState()
        if (keys) {
          if (IS_BROWSER) {
            ric(function () {
              Promise.all(keys.map(function (key) { return cacheItem(key, state[key], {version: opts.version}); }
              )).then(function () {
                if (state.debug) {
                  console.info(("persisted " + (keys.join(', ')) + " because of action " + (action.type)))
                }
              })
            })
          }
        }
        return res
      }; };
      }
    }
  }
}

var getError = function (message, permanent) {
  if ( permanent === void 0 ) permanent = false;

  var err = new Error(message)
  if (permanent) err.permanent = true
  return err
}

var geoErrorArray = [
  'An unknown geolocation error occured',
  'Geolocation permission denied',
  'Geolocation unavailable',
  'Geolocation request timed out'
]

var defaultOpts$2 = {
  timeout: 60000,
  enableHighAccuracy: false,
  persist: true,
  staleAge: 900000, // fifteen minutes
  retryAfter: 60000 // one minute,
}

var geolocation = function (spec) {
  var opts = Object.assign({}, defaultOpts$2, spec)
  return createAsyncResourceBundle$1({
    name: 'geolocation',
    actionBaseType: 'REQUEST_GEOLOCATION',
    getPromise: function () { return new Promise(function (resolve, reject) {
      if (!IS_BROWSER || !navigator.geolocation) {
        reject(getError('Geolocation not supported', true))
      }
      var success = function (position) {
        var res = {}
        var coords = position.coords;
        for (var key in coords) {
          res[key] = coords[key]
        }
        res.timestamp = position.timestamp
        resolve(res)
      }
      var fail = function (ref) {
        var code = ref.code;

        reject(getError(geoErrorArray[code], code === 1))
      }
      var geoOpts = {
        timeout: opts.timeout,
        enableHighAccuracy: opts.enableHighAccuracy
      }
      navigator.geolocation.getCurrentPosition(success, fail, geoOpts)
    }); },
    persist: opts.persist,
    staleAge: opts.staleAge,
    retryAfter: opts.retryAfter
  })
}

var raf =
  (IS_BROWSER && self.requestAnimationFrame) ||
  (function (func) { setTimeout(func, 0) })

var defaults$1 = {
  idleTimeout: 30000,
  idleAction: 'APP_IDLE',
  doneCallback: null
}

var getIdleDispatcher$1 = function (timeout, fn) { return debounce(function () {
  raf(fn)
}, timeout); }

var reactors = function (opts) { return ({
  name: 'reactors',
  extract: 'reactors',
  init: function (store, extracted) {
    opts || (opts = {})
    Object.assign(opts, defaults$1)
    var idleAction = opts.idleAction;
    var idleTimeout = opts.idleTimeout;
    var idleDispatcher
    if (idleTimeout) {
      idleDispatcher = getIdleDispatcher$1(
        idleTimeout,
        function () { return store.dispatch({type: idleAction}); }
      )
    }

    var reactorNames = flattenExtractedToArray(extracted)

    store.reactors = reactorNames

    if (process.env.NODE_ENV !== 'production') {
      reactorNames.forEach(function (name) {
        if (!store[name]) {
          throw Error(("Reactor '" + name + "' not found on the store. Make sure you're defining as selector by that name."))
        }
      })
    }
    store.nextReaction = null

    var cancelIfDone = function () {
      if (!IS_BROWSER && !store.nextReaction && (!store.selectAsyncActive || !store.selectAsyncActive())) {
        idleDispatcher && idleDispatcher.cancel()
        opts.doneCallback && opts.doneCallback()
      }
    }

    var dispatchNext = function () {
      // one at a time
      if (store.nextReaction) {
        return
      }
      // look for the next one
      var action
      store.reactors.some(function (name) {
        store.activeReactor = name
        action = store[name]()
        return action
      })
      if (action) {
        // store it
        store.nextReaction = action
        // let browser chill
        ric(function () {
          var nextReaction = store.nextReaction;
          store.activeReactor = null
          store.nextReaction = null
          if (nextReaction.actionCreator) {
            if (nextReaction.arg) {
              store[nextReaction.actionCreator](nextReaction.arg)
            } else if (nextReaction.args) {
              store[nextReaction.actionCreator].apply(store, nextReaction.args)
            } else {
              store[nextReaction.actionCreator]()
            }
          } else {
            store.dispatch(nextReaction)
          }
          cancelIfDone()
        })
      }
    }

    var callback = function () {
      dispatchNext()
      if (idleDispatcher) {
        idleDispatcher()
        cancelIfDone()
      }
    }

    store.subscribe(callback)
    callback()
  }
}); }

var isDefined = function (thing) { return typeof thing !== 'undefined'; }
var ensureString = function (input) { return typeof input === 'string' ? input : qs.stringify(input); }
var IPRE = /^[0-9\.]+$/
var parseSubdomains = function (hostname) {
  if (IPRE.test(hostname)) return []
  return hostname.split('.').slice(0, -2)
}
var removeLeading = function (char, string) { return string.charAt(0) === char ? string.slice(1) : string; }
var ensureLeading = function (char, string) {
  if (string === char || string === '') {
    return ''
  }
  return string.charAt(0) !== char ? char + string : string
}
var loc = (function () {
  if (!HAS_WINDOW) return {}
  return window.location
})()
var defaults$2 = {
  name: 'url',
  inert: !HAS_WINDOW,
  actionType: 'UPDATE_URL'
}

var url = function (opts) {
  var config = Object.assign({}, defaults$2, opts)
  var actionType = config.actionType

  var selectUrlRaw = function (state) { return state[config.name]; }
  var selectUrlObject = createSelector.createSelector(selectUrlRaw, function (urlState) { return new URL(urlState.url, true); })
  var selectQueryObject = createSelector.createSelector(selectUrlObject, function (urlObj) { return urlObj.query; })
  var selectQueryString = createSelector.createSelector(selectQueryObject, function (queryObj) { return qs.stringify(queryObj); })
  var selectPathname = createSelector.createSelector(selectUrlObject, function (urlObj) { return urlObj.pathname; })
  var selectHash = createSelector.createSelector(selectUrlObject, function (urlObj) { return removeLeading('#', urlObj.hash); })
  var selectHashObject = createSelector.createSelector(selectHash, function (hash) { return qs.parse(hash); })
  var selectSubdomains = createSelector.createSelector(selectUrlObject, function (urlObj) { return parseSubdomains(urlObj.hostname); })

  var doUpdateUrl = function (newState, opts) {
    if ( opts === void 0 ) opts = {replace: false};

    return function (ref) {
    var dispatch = ref.dispatch;
    var getState = ref.getState;

    var state = (typeof newState === 'string') ? { pathname: newState, hash: '', query: '' } : newState
    var url = new URL(selectUrlRaw(getState()).url, true)
    if (isDefined(state.pathname)) url.set('pathname', state.pathname)
    if (isDefined(state.hash)) url.set('hash', ensureString(state.hash))
    if (isDefined(state.query)) url.set('query', ensureString(state.query))
    dispatch({ type: actionType, payload: { url: url.href, replace: opts.replace } })
  };
  }
  var doReplaceUrl = function (url) { return doUpdateUrl(url, {replace: true}); }
  var doUpdateQuery = function (query, opts) {
      if ( opts === void 0 ) opts = {replace: true};

      return doUpdateUrl({ query: ensureString(query) }, opts);
  }
  var doUpdateHash = function (hash, opts) {
      if ( opts === void 0 ) opts = {replace: false};

      return doUpdateUrl({ hash: ensureLeading('#', ensureString(hash)) }, opts);
  }

  return {
    name: config.name,
    init: function (store) {
      if (config.inert) {
        return
      }

      var lastState = store.selectUrlRaw()

      var setCurrentUrl = function () {
        store.doUpdateUrl({
          pathname: loc.pathname,
          hash: loc.hash,
          query: loc.search
        })
      }

      window.addEventListener('popstate', setCurrentUrl)

      store.subscribe(function () {
        var newState = store.selectUrlRaw()
        var newUrl = newState.url
        if (lastState !== newState && newUrl !== loc.href) {
          try {
            window.history[newState.replace ? 'replaceState' : 'pushState']({}, null, newState.url)
          } catch (e) {
            console.error(e)
          }
        }
        lastState = newState
      })
    },
    getReducer: function () {
      var initialState = {
        url: !config.inert && HAS_WINDOW
          ? loc.href
          : '/',
        replace: false
      }

      return function (state, ref) {
        if ( state === void 0 ) state = initialState;
        var type = ref.type;
        var payload = ref.payload;

        if (type === '@@redux/INIT' && typeof state === 'string') {
          return {
            url: state,
            replace: false
          }
        }
        if (type === actionType) {
          return Object.assign({
            url: payload.url || payload,
            replace: !!payload.replace
          })
        }
        return state
      }
    },
    doUpdateUrl: doUpdateUrl,
    doReplaceUrl: doReplaceUrl,
    doUpdateQuery: doUpdateQuery,
    doUpdateHash: doUpdateHash,
    selectUrlRaw: selectUrlRaw,
    selectUrlObject: selectUrlObject,
    selectQueryObject: selectQueryObject,
    selectQueryString: selectQueryString,
    selectPathname: selectPathname,
    selectHash: selectHash,
    selectHashObject: selectHashObject,
    selectSubdomains: selectSubdomains
  }
}

var version = "10.1.1";

var ENABLE = 'ENABLE_DEBUG'
var DISABLE = 'DISABLE_DEBUG'

var debug$1 = {
  name: 'debug',
  reducer: function (state, ref) {
    if ( state === void 0 ) state = HAS_DEBUG_FLAG;
    var type = ref.type;

    if (type === ENABLE) {
      return true
    }
    if (type === DISABLE) {
      return false
    }
    return state
  },
  doEnableDebug: function () { return ({ type: ENABLE }); },
  doDisableDebug: function () { return ({ type: DISABLE }); },
  selectIsDebug: function (state) { return state.debug; },
  init: function (store) {
    if (store.selectIsDebug()) {
      var names = store.bundles.map(function (bundle) { return bundle.name; })
      self.store = store
      var actionCreators = []
      for (var key in store) {
        var item = store[key]
        if (key.indexOf('select') === 0) {
          self[key] = item
        } else if (key.indexOf('do') === 0) {
          self[key] = item
          actionCreators.push(key)
        }
      }
      actionCreators.sort()
      var colorTitle = 'color: #1676D2;'
      var black = 'color: black;'
      var colorGreen = 'color: #4CAF50;'
      var colorOrange = 'color: #F57C00;'
      var normal = 'font-weight: normal;'

      store.logSelectors = self.logSelectors = function () {
        if (!store.selectAll) return
        console.log('%cselectors:', colorGreen, store.selectAll())
      }

      store.logBundles = self.logBundles = function () {
        console.log('%cinstalled bundles:\n  %c%s', colorTitle, black, names.join('\n  '))
      }

      store.logActionCreators = self.logActionCreators = function () {
        console.groupCollapsed('%caction creators', colorOrange)
        actionCreators.forEach(function (name) { return console.log(name); })
        console.groupEnd()
      }

      store.logReactors = self.logReactors = function () {
        console.groupCollapsed('%creactors', colorOrange)
        store.reactors && store.reactors.forEach(function (name) { return console.log(name); })
        console.groupEnd()
      }

      store.logNextReaction = self.logNextReaction = function () {
        var nextReaction = store.nextReaction;
        var activeReactor = store.activeReactor;
        if (nextReaction) {
          console.log(
            ("%cnext reaction:\n  %c" + activeReactor),
            colorOrange,
            black,
            nextReaction
          )
        }
      }

      console.groupCollapsed('%credux bundler v%s', colorTitle, version)
      store.logBundles()
      var exported = []
      for (var key$1 in store) {
        if (key$1.indexOf('select') === 0 || key$1.indexOf('do') === 0) {
          exported.push((key$1 + "()"))
        }
      }
      exported.sort()
      exported.unshift('store')
      console.log(("%cattached to self/window:\n  %c" + (exported.join('\n  '))), colorTitle, black + normal)
      store.logSelectors()
      store.logReactors()
      console.groupEnd()
      store.logNextReaction()
    }
  }
}

var selectAll = {
  name: 'selectAll',
  init: function (store) {
    var selectorNames = Object.keys(store.meta.selectors).sort()
    store.selectAll = function () { return selectorNames.reduce(function (accum, key) {
        accum[selectorNameToValueName(key)] = store[key]()
        return accum
      }, {}); }
  }
}

var appTimeBundle = appTimeBundle$1
var asyncCountBundle = asyncCount
var cachingBundle = caching
var createRouteBundle = createRoutingBundle
var createAsyncResourceBundle = createAsyncResourceBundle$1
var reactorsBundle = reactors
var getIdleDispatcher = getIdleDispatcher$1
var onlineBundle = onlineBundle$1
var urlBundle = url
var debugBundle = debug$1
var composeBundlesRaw = composeBundles$1
var geolocationBundle = geolocation
var selectAllBundle = selectAll
var composeBundles = function () {
  var userBundles = [], len = arguments.length;
  while ( len-- ) userBundles[ len ] = arguments[ len ];

  userBundles || (userBundles = [])
  var bundles = [
    appTimeBundle$1,
    asyncCount,
    onlineBundle$1,
    url(),
    reactors(),
    selectAll,
    debug$1 ].concat( userBundles
  )
  return composeBundles$1.apply(void 0, bundles)
}

exports.appTimeBundle = appTimeBundle;
exports.asyncCountBundle = asyncCountBundle;
exports.cachingBundle = cachingBundle;
exports.createRouteBundle = createRouteBundle;
exports.createAsyncResourceBundle = createAsyncResourceBundle;
exports.reactorsBundle = reactorsBundle;
exports.getIdleDispatcher = getIdleDispatcher;
exports.onlineBundle = onlineBundle;
exports.urlBundle = urlBundle;
exports.debugBundle = debugBundle;
exports.composeBundlesRaw = composeBundlesRaw;
exports.geolocationBundle = geolocationBundle;
exports.selectAllBundle = selectAllBundle;
exports.composeBundles = composeBundles;
exports.createSelector = createSelector.createSelector;
exports.resolveSelectors = createSelector.resolveSelectors;
exports.HAS_DEBUG_FLAG = HAS_DEBUG_FLAG;
exports.HAS_WINDOW = HAS_WINDOW;
exports.IS_BROWSER = IS_BROWSER;
exports.flattenExtractedToObject = flattenExtractedToObject;
exports.flattenExtractedToArray = flattenExtractedToArray;
exports.addGlobalListener = addGlobalListener;
exports.selectorNameToValueName = selectorNameToValueName;
exports.getCachedItem = getCachedItem;
exports.getAllCached = getAllCached;
exports.clearAllCached = clearAllCached;
exports.cacheItem = cacheItem;
